/*
 * exempi/demo - xmpdump.cpp
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */


#include <stdio.h>
#include <stdlib.h>

#include <exempi/xmp.h>


void dump_schema(const char *name, XmpValuePtr value)
{
	const char * v = xmp_value_get_string(value);
	printf("\t%s = ", name);
	if (v != NULL) {
		printf("%s\n", v);
	}
	else {
		printf("<unknown type>\n");
	}
}


void dump_tree(const char *name, XmpSchemaPtr value)
{

	XmpPropertyIterator iter = xmp_schema_get_property_iter(value);
  printf("%s\n", name);
	while(xmp_property_iter_is_valid(iter)) {
		dump_schema(xmp_property_iter_name(iter),
								xmp_property_iter_value(iter));
		xmp_property_iter_next(iter);
	}

	xmp_property_iter_free(iter);
}

int main(int argc, char **argv)
{

	size_t len;
	char * buffer;
	FILE * f;
	size_t rlen;
	XmpPtr xmp;
	XmpTreePtr tree;
	XmpSchemaIterator iter;

	if (argc < 1) {
		fprintf(stderr, "missing arg\n");
	}

	f = fopen(argv[1], "rb");
 	fseek(f, 0, SEEK_END);
	len = ftell(f);
 	fseek(f, 0, SEEK_SET);

	buffer = (char*)malloc(len + 1);
	rlen = fread(buffer, 1, len, f);
	xmp = xmp_new(buffer, len);

	tree = xmp_get_tree(xmp);
	iter = xmp_tree_get_schema_iter(tree);
	
  while (xmp_schema_iter_is_valid(iter)) {
		dump_tree(xmp_schema_iter_name(iter), 
							xmp_schema_iter_value(iter));
		xmp_schema_iter_next(iter);
	}

	xmp_schema_iter_free(iter);
	xmp_free(xmp);

	free(buffer);
	fclose(f);

	return 0;
}
