/*
 * exempi - test1.cpp
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */

#include <stdlib.h>
#include <stdio.h>

#include <iostream>
#include <string>

#include <exempi/xmp.h>
#include <exempi/xmpconsts.h>


#define CHECK_NEQUAL(x,y,z) \
  CHECK(x != y, y, z)

#define CHECK_EQUAL(x,y,z) \
  CHECK(x == y, y, z)

#define CHECK(x,e,y) \
  { check_result((x), e, #x, y); \
	}


void check_result(bool result, void *expected, const char * condition, 
									const char * test)
{
	check_result(result, 0, condition, test);
}


void check_result(bool result, int expected, const char * condition, 
									const char * test)
{
	if (!result) {
		std::cerr << "FAILED: " << condition << " expected: " << expected
							<< " -- " << test << std::endl;
		exit(128);
	}
	std::cerr << "OK: -- " << test << std::endl;

}


int main(int argc, char **argv)
{
	std::string testfile;
	if (argc == 1) {
		// no argument, lets run like we are in "check"
		const char * srcdir = getenv("srcdir");
		
		CHECK_NEQUAL(srcdir, (void*)NULL, "get source dir");
		testfile = srcdir;
		testfile += "/test1.xmp";
	}
	else {
		testfile = argv[1];
	}

	size_t len;
	char * buffer;
	
	FILE * f = fopen(testfile.c_str(), "rb");

	CHECK_NEQUAL(f, (FILE*)NULL, "open file");
	if (f == NULL) {
		exit(128);
	}
 	fseek(f, 0, SEEK_END);
	len = ftell(f);
 	fseek(f, 0, SEEK_SET);

	buffer = (char*)malloc(len + 1);
	/*size_t rlen =*/ fread(buffer, 1, len, f);

	XmpPtr xmp = xmp_new(buffer, len);

	CHECK_NEQUAL(xmp, (XmpPtr)NULL, "xmp packet creation");

	XmpTreePtr tree = xmp_get_tree(xmp);
	
	CHECK_NEQUAL(tree, (XmpTreePtr)NULL, "get xmp tree"); 

	XmpSchemaIterator iter = xmp_tree_get_schema_iter(tree);
	CHECK_NEQUAL(iter, (XmpSchemaIterator)NULL, "schema iterator");
	int i = 0;
	while (xmp_schema_iter_is_valid(iter)) {
		i++;
		xmp_schema_iter_next(iter);
	}
	CHECK_EQUAL(i, 10, "number of schemas");
	xmp_schema_iter_free(iter);

	iter = xmp_tree_get_schema_iter(tree);
	while(xmp_schema_iter_is_valid(iter)) {
		if (strcmp(NS_TIFF, xmp_schema_iter_name(iter)) == 0) {
			break;
		}
		xmp_schema_iter_next(iter);		
	}
	CHECK_EQUAL(xmp_schema_iter_is_valid(iter), true, "found TIFF ns");
	XmpSchemaPtr schema = xmp_schema_iter_value(iter);
	CHECK_NEQUAL(schema, (XmpSchemaPtr)NULL, "retrieve TIFF schema");

	XmpPropertyIterator iter2 = xmp_schema_get_property_iter(schema);
	i = 0;
	while(xmp_property_iter_is_valid(iter2)) {
		i++;
		xmp_property_iter_next(iter2);
	}
	CHECK_EQUAL(i, 5, "count TIFF properties");
	xmp_property_iter_free(iter2);
	
	iter2 = xmp_schema_get_property_iter(schema);
	while(xmp_property_iter_is_valid(iter2)) {
		if (strcmp(xmp_property_iter_name(iter2), "Make") == 0) {
			break;
		}
		xmp_property_iter_next(iter2);
	}
	
	XmpValuePtr value = xmp_property_iter_value(iter2);
	const char* make = xmp_value_get_string(value);
	CHECK_NEQUAL(make, (void*)NULL, "xmp_value_get_string");
	CHECK_EQUAL(strcmp(make, "Canon"), 0, "check value");

	CHECK_EQUAL(strcmp(make, xmp_get_property(xmp, NS_TIFF, "Make")), 
							0, "xmp_get_property"); 

	xmp_property_iter_free(iter2);
	
	xmp_schema_iter_free(iter);

	iter = xmp_tree_get_schema_iter(tree);
	while(xmp_schema_iter_is_valid(iter)) {
		if (strcmp(NS_TPG, xmp_schema_iter_name(iter)) == 0) {
			break;
		}
		xmp_schema_iter_next(iter);		
	}

	CHECK_EQUAL(xmp_schema_iter_is_valid(iter), true, "found TPG ns");
	schema = xmp_schema_iter_value(iter);
	iter2 = xmp_schema_get_property_iter(schema);
	while(xmp_property_iter_is_valid(iter2)) {
		if (strcmp(xmp_property_iter_name(iter2), "MaxPageSize") == 0) {
			break;
		}
		xmp_property_iter_next(iter2);
	}	
	CHECK_EQUAL(xmp_property_iter_is_valid(iter2), true, 
							"found MaxPageSize prop");
	if (xmp_property_iter_is_valid(iter2)) {
		value = xmp_property_iter_value(iter2);
		const char *field = xmp_value_get_field(value, "unit");
		CHECK_NEQUAL(field, (void*)NULL, "get unit field");
		CHECK_EQUAL(strcmp(field, "inches"), 
								0, "checking unit value");
	}
	xmp_property_iter_free(iter2);
	
	xmp_schema_iter_free(iter);
	
	xmp_free(xmp);

	free(buffer);
	fclose(f);
	return 0;
}
